@@ -28,6 +28,7 @@ import typing as T
 if T.TYPE_CHECKING:
     from ..environment import Environment
     from ..mesonlib import MachineChoice
+    from ..utils.core import EnvironOrDict
     from .._typing import ImmutableListProtocol
     from ..build import EnvironmentVariables
 
@@ -124,7 +125,11 @@ class PkgConfigDependency(ExternalDependency):
         p, out, err = Popen_safe(cmd, env=env)
         rc, out, err = p.returncode, out.strip(), err.strip()
         call = ' '.join(cmd)
-        mlog.debug(f"Called `{call}` -> {rc}\n{out}")
+        mlog.debug(f"Called `{call}` -> {rc}")
+        if out:
+            mlog.debug(f'stdout:\n{out}\n-----------')
+        if err:
+            mlog.debug(f'stderr:\n{err}\n-----------')
         return rc, out, err
 
     @staticmethod
@@ -148,7 +153,7 @@ class PkgConfigDependency(ExternalDependency):
         return env
 
     @staticmethod
-    def setup_env(env: T.MutableMapping[str, str], environment: 'Environment', for_machine: MachineChoice,
+    def setup_env(env: EnvironOrDict, environment: 'Environment', for_machine: MachineChoice,
                   uninstalled: bool = False) -> T.Dict[str, str]:
         envvars = PkgConfigDependency.get_env(environment, for_machine, uninstalled)
         env = envvars.get_env(env)
@@ -158,7 +163,7 @@ class PkgConfigDependency(ExternalDependency):
                 mlog.debug(f'env[{key}]: {value}')
         return env
 
-    def _call_pkgbin(self, args: T.List[str], env: T.Optional[T.MutableMapping[str, str]] = None) -> T.Tuple[int, str, str]:
+    def _call_pkgbin(self, args: T.List[str], env: T.Optional[EnvironOrDict] = None) -> T.Tuple[int, str, str]:
         assert isinstance(self.pkgbin, ExternalProgram)
         env = env or os.environ
         env = PkgConfigDependency.setup_env(env, self.env, self.for_machine)
@@ -318,7 +323,8 @@ class PkgConfigDependency(ExternalDependency):
                     continue
                 if self.clib_compiler:
                     args = self.clib_compiler.find_library(lib[2:], self.env,
-                                                           libpaths, self.libtype)
+                                                           libpaths, self.libtype,
+                                                           lib_prefix_warning=False)
                 # If the project only uses a non-clib language such as D, Rust,
                 # C#, Python, etc, all we can do is limp along by adding the
                 # arguments as-is and then adding the libpaths at the end.
